package com.example.serviceImpl;

import com.example.db1.dao.ArchiveDetailDao;
import com.example.db1.dao.ArchiveMasterDao;
import com.example.db1.dao.CommonTableDao;
import com.example.db1.dao.ZdAssortDao;
import com.example.db2.dao.MedicalDao;
import com.example.dto.*;
import com.example.service.ArchiveService;
import com.example.vo.ArchiveMasterVo;
import com.example.vo.GetZdAssortVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.nio.file.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName ArchiveServiceImpl
 * @Description 连州归档业务层
 * @Author linjj
 * @Date 2024/12/13 13:15
 * @Version 1.0
 */
@Service
@Slf4j
public class ArchiveServiceImpl implements ArchiveService {

    @Value("${emr_pdf_files}")
    private String emr_pdf_files;

    @Value("${save_pdf_files}")
    private String save_pdf_files;
    @Autowired
    private MedicalDao medicalDao;
    @Autowired
    private ArchiveMasterDao archiveMasterDao;
    @Autowired
    private ZdAssortDao zdAssortDao;
    @Autowired
    private ArchiveDetailDao archiveDetailDao;
    @Autowired
    private CommonTableDao commonTableDao;


    @Override
    public boolean archive() {
        //获取最早的文件名
        String fileName = getEarlyFileName();
        //若返回空说明没有文件直接return
        if (fileName == null) return false;
        //-----------------------判断该病案是否上传完成代码---------------------------------
        //将文件姓名解析 使用下划线 "_" 分割字符串
        String[] parts = fileName.split("_");
        GetMedicalDto getMedicalDto = new GetMedicalDto();
        getMedicalDto.setNAME(parts[0]);
        getMedicalDto.setSICK_ID(parts[1]);
        getMedicalDto.setRESIDENCE_NO(parts[2]);
        //存储患者基本信息
        AddMasterDto addMasterDto = new AddMasterDto();
        //查询该患者在视图中完成状态是否为1
        List<ArchiveMasterVo> archiveMasterVos = medicalDao.GetMedicalInfo(getMedicalDto);
        if (archiveMasterVos.isEmpty()) {
            log.info("该病案还未上传完成：" + fileName);
            return false;
        }
        //判断该病案是否存在归档数据库存在更新不存在新增,捕获到异常返回false直接return
        if (!updateMedical(fileName, getMedicalDto, archiveMasterVos, addMasterDto)) return false;
        //-----------------------将文件复制代码(将本次查询出来最早的文件备份后操作)---------------------------------
        Path sourcePath = Paths.get(emr_pdf_files + File.separatorChar + fileName); // 源目录路径
        Path targetPath = Paths.get(save_pdf_files + File.separatorChar + fileName); // 目标目录路径
        try {
            copyDirectory(sourcePath, targetPath); // 调用复制目录的方法
            log.info("该病历图像已经复制完成：" + fileName);
        } catch (Exception e) {
            log.error("复制源文件到目标路径下失败,异常处理" + e, e.getMessage());
            return false;
        }
        //将新地址插入数据库
        addArchiveDetail(fileName, addMasterDto);
        return true;
    }

    /**
     * @description: 将新地址插入数据库
     * @params: fileName
     * @params: addMasterDto
     * @author linjj
     * @date: 2024/12/12 16:43
     */
    private void addArchiveDetail(String fileName, AddMasterDto addMasterDto) {
        // 创建File对象新地址
        File newFile = new File(save_pdf_files + File.separatorChar + fileName);
        // 获取目录下的所有文件（包括子目录中的文件）
        File[] newFiles = newFile.listFiles();
        //保存文件的数据
        List<AddDetailDto> addDetailDtos = new ArrayList<AddDetailDto>();
        if (newFiles.length == 0) {
            log.info("新地址目录为空异常处理:" + fileName);
            return;
        }
        for (File file : newFiles) {
            //组织数据
            AddDetailDto addDetailDto = new AddDetailDto();
            addDetailDto.setPDF_PATH(file.getPath());
            addDetailDto.setMasterID(addMasterDto.getId());
            addDetailDto.setUpLoadDateTime(new Date());
            List<GetZdAssortVo> zdAssort = zdAssortDao.getZdAssort();
            String newFileName = file.getName();
            //将文件姓名解析 使用下划线 "_" 分割字符串
            String[] newParts = newFileName.split("_");
            // 调用方法查找name为"Item2"的id
            String assortId = assortIdByName(zdAssort, newParts[0]);
            if (StringUtils.isEmpty(assortId)) {
                assortId = "C7C73CD034B440F6B33A79E382A5610F";
            }
            addDetailDto.setAssortID(assortId);
            addDetailDto.setSource("后台");
            addDetailDto.setSubAssort(newParts[0]);
            addDetailDto.setTitle(newParts[0]);
            addDetailDto.setFlag("0");
            addDetailDto.setSys("2");
            addDetailDto.setSplitName("guiDang");
            addDetailDtos.add(addDetailDto);
        }
        if (archiveDetailDao.addArchiveDetail(addDetailDtos)) {
            log.info("加入文件表完成：" + fileName);
        } else {
            log.info("加入文件表失败异常处理：" + fileName);
        }
    }

    /**
     * @description: 根据分段名称获取
     * @params: zdAssortDto
     * @params: name
     * @return: assortId
     * @author linjj
     * @date: 2024/12/12 16:25
     */
    public static String assortIdByName(List<GetZdAssortVo> zdAssortDto, String name) {
        return zdAssortDto.stream()
                .filter(item -> item.getAssortName().equals(name))
                .map(GetZdAssortVo::getAssortId)
                .findFirst()
                .orElse(null);
    }


    /**
     * @description: 判断该病案是否存在归档数据库存在更新不存在新增
     * @params: fileName
     * @params: getMedicalDto
     * @params: ArchiveMasterVo
     * @return: boolean
     * @author linjj
     * @date: 2024/12/12 12:07
     */
    private boolean updateMedical(String fileName, GetMedicalDto getMedicalDto, List<ArchiveMasterVo> archiveMasterVos, AddMasterDto addMasterDto) {
        //判断是否存在该患者信息基础表
        try {
            GetMasterIdDto getMasterIdDto = new GetMasterIdDto();

            //使用时间生成一个id
            SimpleDateFormat fmt = new SimpleDateFormat("yyyyMMddHHmmssSSS");
            //获取今天日期
            String id = fmt.format(new Date());
            getMasterIdDto.setName(getMedicalDto.getNAME());
            getMasterIdDto.setPatientId(getMedicalDto.getSICK_ID());
            getMasterIdDto.setInpNo(getMedicalDto.getRESIDENCE_NO());
            List<String> masterIds = archiveMasterDao.getMasterId(getMasterIdDto);
            //不存在新增
            if (masterIds.isEmpty()) {
                //组织需要保存归档基础信息表的数据,不存在时使用当前时间为患者id
                addMasterDto.setId(id);
                addMasterDto.setPatientId(archiveMasterVos.get(0).getSICK_ID());
                addMasterDto.setInpNo(archiveMasterVos.get(0).getRESIDENCE_NO());
                addMasterDto.setVisitId(archiveMasterVos.get(0).getVISIT_NUMBER());
                addMasterDto.setName(archiveMasterVos.get(0).getNAME());
                addMasterDto.setSex(archiveMasterVos.get(0).getSEX());
                addMasterDto.setDeptName(archiveMasterVos.get(0).getDEPT_CODE());
                addMasterDto.setDischargeDateTime(archiveMasterVos.get(0).getDISCHARGE_TIME());
                addMasterDto.setArchiveState("已归档");
                addMasterDto.setAdmissionDateTime(archiveMasterVos.get(0).getADMISSION_TIME());
                addMasterDto.setDeptAdmissionTo(archiveMasterVos.get(0).getADMISSION_DEPT_CODE());
                addMasterDto.setCheckDatetime(new Date());
                addMasterDto.setCheckedDatetime(new Date());
                addMasterDto.setDOCTOR_IN_CHARGE(archiveMasterVos.get(0).getDOCTOR());
                addMasterDto.setID_NO(archiveMasterVos.get(0).getID_CARD_NO());
                addMasterDto.setSplitName("guiDang");
                if (archiveMasterDao.addArchiveMaster(addMasterDto)) {
                    log.info("同步基础信息成功:" + fileName);
                }
                //不存是时同样在CommonTable插入新数据
                AddCommonTableDto addCommonTableDto =new AddCommonTableDto();
                addCommonTableDto.setPatientId(id);
                addCommonTableDto.setAdmissId(archiveMasterVos.get(0).getSICK_ID());
                addCommonTableDto.setAdmissTimes(archiveMasterVos.get(0).getVISIT_NUMBER());
                addCommonTableDto.setInpatientNo(archiveMasterVos.get(0).getRESIDENCE_NO());
                addCommonTableDto.setName(archiveMasterVos.get(0).getNAME());
                addCommonTableDto.setSex(archiveMasterVos.get(0).getSEX());
                addCommonTableDto.setAdmissDate(archiveMasterVos.get(0).getADMISSION_TIME());
                addCommonTableDto.setDisDate(archiveMasterVos.get(0).getDISCHARGE_TIME());
                addCommonTableDto.setDisDept(archiveMasterVos.get(0).getDEPT_CODE());
                addCommonTableDto.setAttending(archiveMasterVos.get(0).getDOCTOR());
                addCommonTableDto.setDisDeptName(archiveMasterVos.get(0).getDETP_NAME());
                addCommonTableDto.setIdCard(archiveMasterVos.get(0).getID_CARD_NO());
                addCommonTableDto.setAdmissDoctor(archiveMasterVos.get(0).getDOCTOR());
                addCommonTableDto.setSplitName("guiDang");
                addCommonTableDto.setPh("-");
                addCommonTableDto.setGdh("-");
                if (commonTableDao.addCommonTable(addCommonTableDto)) {
                    log.info("同步基础信息到CommonTable表成功:" + fileName);
                }
                //存在时先删除后使用原masterIds赋值
            } else {
                addMasterDto.setId(masterIds.get(0));
                addMasterDto.setPatientId(archiveMasterVos.get(0).getSICK_ID());
                addMasterDto.setInpNo(archiveMasterVos.get(0).getRESIDENCE_NO());
                addMasterDto.setVisitId(archiveMasterVos.get(0).getVISIT_NUMBER());
                addMasterDto.setName(archiveMasterVos.get(0).getNAME());
                addMasterDto.setSex(archiveMasterVos.get(0).getSEX());
                addMasterDto.setDeptName(archiveMasterVos.get(0).getDEPT_CODE());
                addMasterDto.setDischargeDateTime(archiveMasterVos.get(0).getDISCHARGE_TIME());
                addMasterDto.setArchiveState("已归档");
                addMasterDto.setAdmissionDateTime(archiveMasterVos.get(0).getADMISSION_TIME());
                addMasterDto.setDeptAdmissionTo(archiveMasterVos.get(0).getADMISSION_DEPT_CODE());
                addMasterDto.setCheckDatetime(new Date());
                addMasterDto.setCheckedDatetime(new Date());
                addMasterDto.setDOCTOR_IN_CHARGE(archiveMasterVos.get(0).getDOCTOR());
                addMasterDto.setID_NO(archiveMasterVos.get(0).getID_CARD_NO());
                addMasterDto.setSplitName("guiDang");
                if (archiveMasterDao.delMasterById(masterIds.get(0))) {
                    if (archiveMasterDao.addArchiveMaster(addMasterDto)) {
                        log.info("同步基础信息成功:" + fileName);
                    }
                }
                //存在时同样删除CommonTable数据
                AddCommonTableDto addCommonTableDto =new AddCommonTableDto();
                addCommonTableDto.setPatientId(masterIds.get(0));
                addCommonTableDto.setAdmissId(archiveMasterVos.get(0).getSICK_ID());
                addCommonTableDto.setAdmissTimes(archiveMasterVos.get(0).getVISIT_NUMBER());
                addCommonTableDto.setInpatientNo(archiveMasterVos.get(0).getRESIDENCE_NO());
                addCommonTableDto.setName(archiveMasterVos.get(0).getNAME());
                addCommonTableDto.setSex(archiveMasterVos.get(0).getSEX());
                addCommonTableDto.setAdmissDate(archiveMasterVos.get(0).getADMISSION_TIME());
                addCommonTableDto.setDisDate(archiveMasterVos.get(0).getDISCHARGE_TIME());
                addCommonTableDto.setDisDept(archiveMasterVos.get(0).getDEPT_CODE());
                addCommonTableDto.setAttending(archiveMasterVos.get(0).getDOCTOR());
                addCommonTableDto.setDisDeptName(archiveMasterVos.get(0).getDETP_NAME());
                addCommonTableDto.setIdCard(archiveMasterVos.get(0).getID_CARD_NO());
                addCommonTableDto.setAdmissDoctor(archiveMasterVos.get(0).getDOCTOR());
                addCommonTableDto.setSplitName("guiDang");
                addCommonTableDto.setPh("-");
                addCommonTableDto.setGdh("-");
                if (commonTableDao.delCommonTableById(masterIds.get(0))){
                    if (commonTableDao.addCommonTable(addCommonTableDto)) {
                        log.info("同步基础信息到CommonTable表成功:" + fileName);
                    }
                }
            }
        } catch (Exception e) {
            log.info("同步基础信息失败请异常处理:" + fileName + "异常信息:" + e, e.getMessage());
            return false;
        }
        return true;
    }

    /**
     * @description: 获取最早的文件名
     * @return: fileName
     * @author linjj
     * @date: 2024/12/12 11:36
     */
    private String getEarlyFileName() {
        // 创建File对象，代表指定的目录
        File directory = new File(emr_pdf_files);
        // 确保目录存在
        if (!directory.isDirectory()) {
            System.out.println("目录不存在");
            log.info("目录不存在");
            return null;
        }
        // 获取目录下所有文件和文件夹
        File[] files = directory.listFiles(file -> file.isDirectory());
        if (files.length == 0) {
            log.info("文件夹为空");
            return null;
        }
        // 初始化最早文件夹和最早时间戳
        File oldestFile = null;
        long oldestTime = Long.MAX_VALUE;
        // 遍历所有文件夹
        for (File folder : files) {
            // 获取文件夹的最后修改时间
            long lastModified = folder.lastModified();
            // 如果当前文件夹的修改时间更早，则更新最早文件夹和时间戳
            if (lastModified < oldestTime) {
                oldestTime = lastModified;
                oldestFile = folder;
            }
        }
        // 输出创建时间最早的文件信息
        if (oldestFile == null) {
            log.info("没有找到复合条件的文件信息");
            return null;
        }
        //最早的文件名
        String fileName = oldestFile.getName();
        log.info("当前操作文件夹信息为：" + fileName);
        return fileName;
    }

    /**
     * @description: 复制目录方法
     * @params: source
     * @params: target
     * @author linjj
     * @date: 2024/12/12 16:06
     */
    public static void copyDirectory(Path source, Path target) throws Exception {
        // 如果目标目录不存在，则创建它
        if (!Files.exists(target)) {
            Files.createDirectories(target);
        }
        // 使用DirectoryStream遍历源目录中的所有文件和子目录
        try (DirectoryStream<Path> directoryStream = Files.newDirectoryStream(source)) {
            for (Path path : directoryStream) {
                Path targetPath = target.resolve(source.relativize(path));
                //存在相同文件时REPLACE_EXISTING会替换掉相同的文件夹
                Files.copy(path, targetPath, StandardCopyOption.REPLACE_EXISTING);
                //在复制完成后删除对应的源文件
                Files.delete(path); // 删除源文件
            }
            //复制成功后将源地址文件删除
            Files.delete(source);
        }
    }
}
